# -*- coding: utf8 -*-
from __future__ import unicode_literals
from collections import OrderedDict

from driverng.astm import (
    AstmMessageV2,
    HprimString,
    HprimMultiple,
    HprimList,
)
from driverng.drivers.sysmex.base import (
    SysmexSegmentH,
    SysmexSegmentP,
    SysmexSegmentOBR,
    SysmexSegmentOBX,
    AstmSysmexBase,
)


class SysmexPlusSegmentH(SysmexSegmentH):

    version = HprimString(13, default="SUITplus1.0")


class SysmexPlusSegmentP(SysmexSegmentP):

    # Seulement en SUIT+
    diag_code = HprimString((19, 1), max_length=20)
    diag_name = HprimString((19, 2), max_length=80)


class SysmexPlusSegmentOBR(SysmexSegmentOBR):

    # Order Process Code
    # L: Add orders
    # C: Cancel order
    code_action = HprimString(12, max_length=1)

    # CSF: CSF
    # ASCITES: Ascites
    # PLEURAL: Pleural Fluid
    # CAPD: CAPD Fluid
    # SYNOVIA: Synovial Fluid
    body_fluid = HprimString((16, 2))

    doctor_code = HprimString((17, 1), max_length=20)
    doctor_name = HprimString((17, 2), max_length=80)

    clinic_code = HprimString((19, 1), max_length=20)
    clinic_name = HprimString((19, 2), max_length=80)

    ward_code = HprimString((20, 1), max_length=20)
    ward_name = HprimString((20, 2), max_length=80)


class SysmexPlusSegmentOBX(SysmexSegmentOBX):

    # Only in SUIT+
    # R: Research
    # D: Diagnostic
    # S: Service
    parameter_classification = HprimString(5, max_length=1)

    # c is set when a parameter was re-calculated
    # & is set:
    # - When WBC, LYMPH# and LYMPH% are compensated by NRBC
    # - When NEUT# and NEUT% are compensated by IG or NRBC
    # - When WBC is reported by using WBC-D and PLT is reported by using PLT-O
    # or PLT-F
    comment_code = HprimString((6, 2), max_length=3)

    # Only in SUIT+
    reference_ranges = HprimString(8, max_length=100)

    # Reagent information (Only in SUIT+)
    reagents = HprimList(10, basefield=HprimMultiple(
        10,
        code=HprimString(1),
        lot=HprimString(2),
        serial=HprimString(3),
    ))

    # Only in SUIT+
    # In case results are produced by the manual differentiation or a result
    # was changed manually by the user -> OBX-16 is empty
    prod_analyser_type = HprimString((16, 1), max_length=25)
    prod_instrument_id = HprimString((16, 2), max_length=25)
    prod_rack = HprimString((16, 3), max_length=25)
    prod_tube = HprimString((16, 4), max_length=25)


class AstmSysmexPlus(AstmSysmexBase):
    SPEC = {
        "code": "sysmex_suit_plus",
        "label": "Sysmex SUIT+",
    }

    SEGMENT_CLASS_MAP = AstmSysmexBase.SEGMENT_CLASS_MAP.copy()
    SEGMENT_CLASS_MAP.update(**{
        "H": SysmexPlusSegmentH,
        "P": SysmexPlusSegmentP,
        "OBR": SysmexPlusSegmentOBR,
        "OBX": SysmexPlusSegmentOBX,
    })
    HANDLE_DELETED = True
    OPTIONS = AstmSysmexBase.OPTIONS + [
        AstmMessageV2.BooleanOption(
            "sysmex_instrument_id_seul",
            label="Prendre l'id instrument seul comme code analyseur",
            default=False,
        ),
        AstmMessageV2.Option(
            "analyseurs_a_ignorer",
            label="Liste des codes analyseurs séparés par des ';' pour ignorer le rack",
            default="",
        ),
        AstmMessageV2.BooleanOption(
            "sysmex_suitplus_format_rack",
            label="Formatage du rack pour le SUITPLUS <instrument>_<rack>",
            default=False,
        ),
        AstmMessageV2.BooleanOption(
            "sysmex_suitplus_qc_no_analyseur",
            label="Ne pas formater le code analyseur pour les QC et prendre l'ID de l'instrument présent dans le segment S",
            default=False,
        ),
    ]

    def __init__(self, sysmex_instrument_id_seul=False, analyseurs_a_ignorer="",
                 sysmex_suitplus_format_rack=False,
                 sysmex_suitplus_qc_no_analyseur=False, **kwargs):
        super(AstmSysmexPlus, self).__init__(**kwargs)
        self.instrument_id_seul = sysmex_instrument_id_seul
        self.analyseurs_a_ignorer = analyseurs_a_ignorer
        self.sysmex_suitplus_format_rack = sysmex_suitplus_format_rack
        self.sysmex_suitplus_qc_no_analyseur = sysmex_suitplus_qc_no_analyseur

    @staticmethod
    def get_lots_reactifs(seg_obx, reagent_type):
        lots = []
        for reagent in seg_obx.reagents:
            lot = {
                "lot": reagent.lot,
                "name": reagent.code,
                "type": reagent_type
            }
            if lot not in lots:
                lots.append(lot)
        return lots

    @staticmethod
    def ignore_segment_obx(seg_obx):
        # On ne garde que les résultats finaux
        if seg_obx.status == "F":
            return False
        else:
            return True

    def get_analyseur(self, analyseur, seg_obx):
        if self.instrument_id_seul:
            return seg_obx.prod_instrument_id
        return "{}-{}".format(
            seg_obx.prod_analyser_type,
            seg_obx.prod_instrument_id,
        )

    def get_analyseur_qc(self, message, seg_s):
        if self.sysmex_suitplus_qc_no_analyseur:
            return None
        elif self.instrument_id_seul:
            return seg_s.instrument_id
        elif not self.instrument_id_seul and not self.sysmex_suitplus_qc_no_analyseur:
            return "{}-{}".format(
                seg_s.instrument_id,
                seg_s.instrument_name,
            )

    def get_analyseur_rack(self, seg_obr):
        rack = None
        for seg_obx in seg_obr.results:
            if seg_obx.prod_rack and seg_obx.prod_tube:
                rack = "{}-{}".format(
                    seg_obx.prod_rack,
                    seg_obx.prod_tube,
                )
        return None, rack

    def get_tube_graphe(self, commentaire, executant):
        return executant, commentaire.rsplit('-')[-1].split('_', 1)[1]

    def get_alarmes(self, seg_obx):
        alarmes = seg_obx.abnormal_flags[:]
        if (seg_obx.test_code, seg_obx.comment_code) == ("PLT", "&"):
            alarmes.append("O")
        elif (seg_obx.test_code, seg_obx.comment_code) == ("WBC", "&"):
            alarmes.append("D")
        return alarmes

    def _decode_patients(self, worklist, message):
        super(AstmSysmexPlus, self)._decode_patients(worklist, message)
        for seg_p in message.patients:
            for seg_obr in seg_p.orders:
                indexes = {}
                rack_modifie = ""
                analyseur, rack = self.get_analyseur_rack(seg_obr)
                for seg_obx in seg_obr.results:
                    instrument = self.get_analyseur(analyseur, seg_obx)
                if self.sysmex_suitplus_format_rack:
                    rack_modifie = "{}_{}".format(seg_obx.prod_analyser_type, rack)
                if self.analyseurs_a_ignorer and instrument in self.analyseurs_a_ignorer:
                    worklist.get_or_create(sample_id=seg_obr.tube_executant).rack = None
                elif rack_modifie:
                    worklist.get_or_create(sample_id=seg_obr.tube_executant).rack = rack_modifie
                rack_resu = False
                for seg_obx in seg_obr.results:
                    if not rack_resu and (rack or rack_modifie):
                        worklist.get_or_create(sample_id=seg_obr.tube_executant).results.add(
                            code="RACK",
                            value=rack_modifie if rack_modifie else rack,
                        )
                        rack_resu = True
                    if seg_obx.comment_code and seg_obx.comment_code != "&":
                        # Traitement des commentaires des morphologies dans une
                        # réponse automate dédiée par test
                        # Le commentaire morphologie est présent dans le champ
                        # OBX.6.2
                        worklist.get_or_create(sample_id=seg_obr.tube_executant).results.add(
                            code="{}_COMMENT".format(seg_obx.test_code),
                            value=seg_obx.comment_code,
                            date=seg_obx.date,
                            instrument=instrument,
                        )
                commentaires = OrderedDict()
                index = index_rbc = index_wbc = index_plt = 1
                for seg_c in seg_obr.commentaires:
                    if seg_c.code2 == "MORPH" and seg_c.code1 in ("WBC", "RBC", "PLT"):
                        code = '_'.join([seg_c.code1, seg_c.code2])
                        commentaires.setdefault(code, []).append(seg_c.commentaire)

                        # Résultat multiples par résultat RBC, WBC, PLT avec un index
                        if seg_c.code1 == "RBC":
                            sample = worklist.get_or_create(
                                type="patient",
                                sample_id=seg_obr.tube_executant,
                            )
                            sample.results.add(
                                code="{}{}".format('_'.join([seg_c.code1, seg_c.code2]), index_rbc),
                                value=seg_c.commentaire,
                            )
                            index_rbc += 1
                        elif seg_c.code1 == "WBC":
                            sample = worklist.get_or_create(
                                type="patient",
                                sample_id=seg_obr.tube_executant
                            )
                            sample.results.add(
                                code="{}{}".format('_'.join([seg_c.code1, seg_c.code2]), index_wbc),
                                value=seg_c.commentaire,
                            )
                            index_wbc += 1
                        elif seg_c.code1 == "PLT":
                            sample = worklist.get_or_create(
                                type="patient",
                                sample_id=seg_obr.tube_executant
                            )
                            sample.results.add(
                                code="{}{}".format('_'.join([seg_c.code1, seg_c.code2]), index_plt),
                                value=seg_c.commentaire,
                            )
                            index_plt += 1
                    else:
                        code_canal = '_'.join([seg_c.code1, seg_c.code2])
                        indexes.setdefault(code_canal, 1)
                        sample = worklist.get_or_create(
                            type="patient",
                            sample_id=seg_obr.tube_executant
                        )
                        sample.results.add(
                            code="{}{}".format(code_canal, indexes[code_canal]),
                            value=seg_c.commentaire,
                        )
                for seg_c in seg_obr.commentaires:
                    if seg_c.code1 in ("WBC", "RBC"):
                        code_canal = "WBCRBC_{}".format(seg_c.code2,)
                        worklist.get_or_create(sample_id=seg_obr.tube_executant).results.add(
                            code="{}{}".format(code_canal, index),
                            value=seg_c.commentaire,
                        )
                        index += 1
                for code, commentaire in commentaires.items():
                    worklist.get_or_create(
                        sample_id=seg_obr.tube_executant
                    ).results.add(
                        code=code,
                        value=";".join(commentaire),
                    )
